#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_networks_moves
short_description: Resource module for organizations _networks _moves
description:
  - Manage operation create of the resource organizations _networks _moves. - > Move
    networks from one organization to another. For more information and caveats around
    network move limitations see the Network Portability documentation.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  network:
    description: Network to be moved.
    suboptions:
      id:
        description: ID of the network to move.
        type: str
    type: dict
  organizationId:
    description: OrganizationId path parameter. Organization ID.
    type: str
  organizations:
    description: Organizations involved in the network move.
    suboptions:
      target:
        description: The organization to which the network is moving.
        suboptions:
          id:
            description: ID of the organization to which the network is moving.
            type: str
        type: dict
    type: dict
  simulate:
    description: If true, simulates the network move and validates the operation without
      committing changes. The network will remain in the source organization.
    type: bool
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations createNetworkMove
    description: Complete reference of the createNetworkMove API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-move
notes:
  - SDK Method used are
    organizations.Organizations.create_network_move,
  - Paths used are
    post /organizations/{organizationId}/networks/moves,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.organizations_networks_moves:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    network:
      id: N_569142402909112097
    organizationId: string
    organizations:
      target:
        id: '146308'
    simulate: false
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "moveId": "string",
      "initiator": {
        "admin": {
          "id": "string"
        }
      },
      "organizations": {
        "source": {
          "id": "string"
        },
        "target": {
          "id": "string"
        }
      },
      "network": {
        "id": "string"
      },
      "createdAt": "string",
      "lastUpdatedAt": "string",
      "result": {
        "status": "string",
        "reason": "string"
      }
    }
"""
