#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: devices_sensor_commands
short_description: Resource module for devices _sensor _commands
description:
  - Manage operation create of the resource devices _sensor _commands.
  - Sends a command to a sensor.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  operation:
    description: Operation to run on the sensor. 'enableDownstreamPower', 'disableDownstreamPower',
      and 'cycleDownstreamPower' turn power on/off to the device that is connected
      downstream of an MT40 power monitor. 'refreshData' causes an MT15 or MT40 device
      to upload its latest readings so that they are immediately available in the
      Dashboard API.
    type: str
  serial:
    description: Serial path parameter.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for sensor createDeviceSensorCommand
    description: Complete reference of the createDeviceSensorCommand API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-device-sensor-command
notes:
  - SDK Method used are
    sensor.Sensor.create_device_sensor_command,
  - Paths used are
    post /devices/{serial}/sensor/commands,
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.devices_sensor_commands:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    operation: disableDownstreamPower
    serial: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "commandId": "string",
      "createdAt": "string",
      "completedAt": "string",
      "createdBy": {
        "adminId": "string",
        "name": "string",
        "email": "string"
      },
      "operation": "string",
      "status": "string",
      "errors": [
        "string"
      ]
    }
"""
